@@ -1,5 +1,5 @@
 package Archive::BagIt;
-our $VERSION = '0.060'; # VERSION
+our $VERSION = '0.061'; # VERSION
 use strict;
 use warnings;
 use utf8;
@@ -9,7 +9,7 @@ our $DEBUG=0;
 use Encode qw(decode);
 use File::Find;
 use Data::Dumper;
-#use Data::Printer;
+
 
 sub new {
   my ($class,$bag_path) = @_;
@@ -105,7 +105,7 @@ sub _write_baginfo {
     use POSIX;
     my($self, $bagit, %param) = @_;
     open(my $BAGINFO, ">", $bagit."/bag-info.txt") or die("Can't open $bagit/bag-info.txt for writing: $!");
-    $param{'Bagging-Date'} = POSIX::strftime("%F", gmtime(time));
+    $param{'Bagging-Date'} = POSIX::strftime("%Y-%m-%d", gmtime(time));
     $param{'Bag-Software-Agent'} = 'Archive::BagIt <http://search.cpan.org/~rjeschmi/Archive-BagIt>';
     while(my($key, $value) = each(%param)) {
         print($BAGINFO "$key: $value\n");
@@ -386,7 +386,7 @@ Archive::BagIt
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 SYNOPSIS
 
@@ -415,7 +415,7 @@ Archive::BagIt
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 WARNING
 
@@ -554,11 +554,6 @@ The latest version of this module is available from the Comprehensive Perl
 Archive Network (CPAN). Visit L<http://www.perl.com/CPAN/> to find a CPAN
 site near you, or see L<https://metacpan.org/module/Archive::BagIt/>.
 
-=head1 SOURCE
-
-The development version is on github at L<https://github.com/Archive-BagIt>
-and may be cloned from L<git://github.com/Archive-BagIt.git>
-
 =head1 BUGS AND LIMITATIONS
 
 You can make new bug reports, and view existing ones, through the
