@@ -15,210 +15,15 @@ use Carp;
 use List::Util qw(uniq);
 use POSIX qw(strftime);
 
-our $VERSION = '0.060'; # VERSION
+our $VERSION = '0.061'; # VERSION
+
+# ABSTRACT: The common base for Archive::BagIt. This is the module for experts. ;)
 
 use Sub::Quote;
 
 my $DEBUG=0;
 
 
-has 'parallel' => ( # used for parallel verify, only usefull if bagits with many files expected!
-    is        => 'rw',
-    predicate => 'has_parallel',
-    default   => undef,
-);
-
-
-has 'bag_path' => (
-    is => 'rw',
-);
-
-has 'bag_path_arr' => (
-    is => 'ro',
-    lazy => 1,
-    builder => '_build_bag_path_arr',
-);
-
-has 'metadata_path' => (
-    is=> 'ro',
-    lazy => 1,
-    builder => '_build_metadata_path',
-);
-
-sub _build_metadata_path {
-    my ($self) = @_;
-    return $self->bag_path;
-}
-
-
-has 'metadata_path_arr' => (
-    is =>'ro',
-    lazy => 1,
-    builder => '_build_metadata_path_arr',
-);
-
-has 'rel_metadata_path' => (
-    is => 'ro',
-    lazy => 1,
-    builder => '_build_rel_metadata_path',
-);
-
-has 'payload_path' => (
-    is => 'ro',
-    lazy => 1,
-    builder => '_build_payload_path',
-);
-
-sub _build_payload_path {
-    my ($self) = @_;
-    return $self->bag_path."/data";
-}
-
-has 'payload_path_arr' => (
-    is => 'ro',
-    lazy => 1,
-    builder => '_build_payload_path_arr',
-);
-
-has 'rel_payload_path' => (
-    is => 'ro',
-    lazy => 1,
-    builder => '_build_rel_payload_path',
-);
-
-has 'checksum_algos' => (
-    is => 'ro',
-    lazy => 1,
-    builder => '_build_checksum_algos',
-);
-
-has 'bag_version' => (
-    is       => 'ro',
-    lazy     => 1,
-    builder  => '_build_bag_version',
-);
-
-has 'bag_encoding' => (
-    is      => 'ro',
-    lazy    => 1,
-    builder => '_build_bag_encoding',
-);
-
-has 'bag_info' => (
-    is        => 'rw',
-    lazy      => 1,
-    builder   => '_build_bag_info',
-    predicate => 1
-);
-
-has 'errors' => (
-    is      => 'rw',
-    lazy    => 1,
-    builder => sub { my $self = shift; return [];},
-);
-
-# bag_info_by_key()
-sub bag_info_by_key {
-    my ($self, $searchkey) = @_;
-    my $info = $self->bag_info();
-    if (defined $searchkey) {
-        foreach my $entry (@{$info}) {
-            if (exists $entry->{$searchkey}) {
-                return $entry->{$searchkey};
-            }
-        }
-    }
-    return;
-}
-
-
-sub _replace_bag_info_by_first_match {
-    my ($self, $searchkey, $newvalue) = @_;
-    my $info = $self->bag_info();
-    if (defined $searchkey) {
-        if ($searchkey =~ m/:/) { croak "key should not contain a colon! (searchkey='$searchkey')"; }
-        my $size = scalar( @{$info});
-        for (my $idx=0; $idx< $size; $idx++) {
-            my %entry = %{ $info->[$idx] };
-            my ($key, $value) = each %entry;
-            if ((defined $key) && ($key eq $searchkey)) {
-                $info->[$idx] = {$searchkey => $newvalue};
-                return $idx;
-            }
-        }
-    }
-    return;
-}
-
-sub _add_or_replace_bag_info {
-    my ($self, $searchkey, $newvalue) = @_;
-    if (defined $searchkey) {
-        if ($searchkey =~ m/:/) { croak "key should not contain a colon! (searchkey='$searchkey')"; }
-        if (defined $self->{bag_info}) {
-            my $idx = $self->_replace_bag_info_by_first_match( $searchkey, $newvalue);
-            if (defined $idx) { return $idx;}
-        }
-        push @{$self->{bag_info}}, {$searchkey => $newvalue};
-        return -1;
-    }
-}
-
-
-has 'forced_fixity_algorithm' => (
-    is   => 'ro',
-    lazy => 1,
-    builder  => '_build_forced_fixity_algorithm',
-);
-
-has 'bag_checksum' => (
-    is => 'ro',
-    lazy => 1,
-    builder => '_build_bag_checksum',
-);
-
-has 'manifest_files' => (
-    is => 'ro',
-    lazy => 1,
-    builder => '_build_manifest_files',
-);
-
-has 'tagmanifest_files' => (
-    is => 'ro',
-    lazy => 1,
-    builder => '_build_tagmanifest_files',
-);
-
-
-
-has 'payload_files' => ( # relatively to bagit base
-    is => 'ro',
-    lazy => 1,
-    builder => '_build_payload_files',
-);
-
-has 'non_payload_files' => (
-    is=>'ro',
-    lazy => 1,
-    builder => '_build_non_payload_files',
-);
-
-has 'plugins' => (
-    is=>'rw',
-    #isa=>'HashRef',
-);
-
-has 'manifests' => (
-    is      => 'rw',
-    lazy    => 1,
-    builder => '_build_manifests'
-    #isa=>'HashRef',
-);
-
-has 'algos' => (
-    is=>'rw',
-    #isa=>'HashRef',
-);
-
 
 around 'BUILDARGS' , sub {
     my $orig = shift;
@@ -230,11 +35,414 @@ around 'BUILDARGS' , sub {
     }
 };
 
+
 sub BUILD {
     my ($self, $args) = @_;
     return $self->load_plugins(("Archive::BagIt::Plugin::Manifest::MD5", "Archive::BagIt::Plugin::Manifest::SHA512"));
 }
 
+###############################################
+
+
+has 'parallel' => ( # used for parallel verify, only usefull if bagits with many files expected!
+    is        => 'rw',
+    predicate => 'has_parallel',
+    default   => undef,
+);
+
+###############################################
+
+
+has 'bag_path' => (
+    is => 'rw',
+);
+
+###############################################
+
+has 'bag_path_arr' => (
+    is => 'ro',
+    lazy => 1,
+    builder => '_build_bag_path_arr',
+);
+
+###############################################
+
+
+has 'metadata_path' => (
+    is=> 'ro',
+    lazy => 1,
+    builder => '_build_metadata_path',
+);
+
+sub _build_metadata_path {
+    my ($self) = @_;
+    return $self->bag_path;
+}
+
+###############################################
+
+has 'metadata_path_arr' => (
+    is =>'ro',
+    lazy => 1,
+    builder => '_build_metadata_path_arr',
+);
+
+
+###############################################
+
+has 'rel_metadata_path' => (
+    is => 'ro',
+    lazy => 1,
+    builder => '_build_rel_metadata_path',
+);
+
+###############################################
+
+
+has 'payload_path' => (
+    is => 'ro',
+    lazy => 1,
+    builder => '_build_payload_path',
+);
+
+sub _build_payload_path {
+    my ($self) = @_;
+    return $self->bag_path."/data";
+}
+
+###############################################
+
+has 'payload_path_arr' => (
+    is => 'ro',
+    lazy => 1,
+    builder => '_build_payload_path_arr',
+);
+
+###############################################
+
+has 'rel_payload_path' => (
+    is => 'ro',
+    lazy => 1,
+    builder => '_build_rel_payload_path',
+);
+
+###############################################
+
+
+has 'checksum_algos' => (
+    is => 'ro',
+    lazy => 1,
+    builder => '_build_checksum_algos',
+);
+
+###############################################
+
+
+has 'bag_version' => (
+    is       => 'ro',
+    lazy     => 1,
+    builder  => '_build_bag_version',
+);
+
+###############################################
+
+
+has 'bag_encoding' => (
+    is      => 'ro',
+    lazy    => 1,
+    builder => '_build_bag_encoding',
+);
+
+###############################################
+
+
+has 'bag_info' => (
+    is        => 'rw',
+    lazy      => 1,
+    builder   => '_build_bag_info',
+    predicate => 1
+);
+
+###############################################
+
+
+has 'errors' => (
+    is      => 'ro',
+    lazy    => 1,
+    builder => sub { my $self = shift; return [];},
+);
+
+###############################################
+
+
+has 'digest_callback' => (
+    is      => 'ro',
+    lazy    => 1,
+    builder => sub {
+        my $sub = sub {
+            my ($digestobj, $filename) = @_;
+            open(my $fh, "<:raw", "$filename") or croak ("Cannot open $filename, $!");
+            binmode($fh);
+            my $digest = $digestobj->get_hash_string($fh);
+            close $fh || croak("could not close file '$filename', $!");
+            return $digest;
+        };
+        return $sub;
+    }
+);
+
+###############################################
+
+
+sub get_baginfo_values_by_key {
+    my ($self, $searchkey) = @_;
+    my $info = $self->bag_info();
+    my @values;
+    if (defined $searchkey) {
+        my $lc_flag = $self->is_baginfo_key_reserved( $searchkey );
+        foreach my $entry (@{ $info }) {
+            my ($key, $value) = %{ $entry };
+            if ( __case_aware_compare_for_baginfo( $key, $searchkey, $lc_flag) ) {
+                push @values, $value;
+            }
+        }
+    }
+    return @values if (scalar(@values) > 0);
+    return;
+}
+
+###############################################
+
+
+sub is_baginfo_key_reserved_as_uniq {
+    my ($self, $searchkey) = @_;
+    return $searchkey =~ m/(Bagging-Date)|(Bag-Size)|(Payload-Oxum)|(Bag-Group-Identifier)|(Bag-Count)/i;
+}
+
+###############################################
+
+
+sub is_baginfo_key_reserved {
+    my ($self, $searchkey) = @_;
+    my @reserved = qw(
+        Source-Organization
+        Organisation-Adress
+        Contact-Name
+        Contact-Phone
+        Contact-Email
+        External-Description
+        Bagging-Date
+        External-Identifier
+        Bag-Size
+        Payload-Oxum
+        Bag-Group-Identifier
+        Bag-Count
+        Internal-Sender-Identifier
+        Internal-Sender-Description
+    );
+    return List::Util::any { (lc $searchkey) eq (lc $_) } @reserved;
+}
+
+###############################################
+
+sub __case_aware_compare_for_baginfo {
+    my ($internal_key, $search_key, $lc_flag) = @_;
+    return (defined $internal_key) && (
+        ( $lc_flag && ((lc $internal_key) eq (lc $search_key)) ) # for reserved keys use caseinsensitive search
+            ||
+            ( (!$lc_flag) && ($internal_key eq $search_key) ) # for other keys sensitive search
+    )
+}
+
+###############################################
+
+sub _find_baginfo_idx {
+    my ($self, $searchkey) = @_;
+    if (defined $searchkey) {
+        if ($searchkey =~ m/:/) {croak "key should not contain a colon! (searchkey='$searchkey')";}
+        my $info = $self->bag_info();
+        my $size = scalar(@{$info});
+        my $lc_flag = $self->is_baginfo_key_reserved($searchkey);
+        for (my $idx = 0; $idx < $size; $idx++) {
+            my %entry = %{$info->[$idx]};
+            my ($key, $value) = %entry;
+            if (__case_aware_compare_for_baginfo($key, $searchkey, $lc_flag)) {
+                return $idx;
+            }
+        }
+    }
+    return;
+}
+###############################################
+
+
+sub verify_baginfo {
+    my ($self) = @_;
+    my %keys;
+    my $info = $self->bag_info();
+    my $ret = 1;
+    if (defined $info) {
+        foreach my $entry (@{$self->bag_info()}) {
+            my ($key, $value) = %{$entry};
+            if ($self->is_baginfo_key_reserved($key)) {
+                $keys{ lc $key }++;
+            }
+            else {
+                $keys{ $key }++
+            }
+        }
+        foreach my $key (keys %keys) {
+            if ($self->is_baginfo_key_reserved_as_uniq($key)) {
+                if ($keys{$key} > 1) {
+                    push @{$self->{errors}}, "Baginfo key '$key' exists $keys{$key}, but should be uniq!";
+                    $ret = undef;
+                }
+            }
+        }
+    }
+    return $ret;
+}
+
+###############################################
+
+
+sub delete_baginfo_by_key {
+    my ($self, $searchkey) = @_;
+    my $idx = $self->_find_baginfo_idx($searchkey);
+    if (defined $idx) {
+        delete $self->{bag_info}[$idx];
+    }
+}
+
+###############################################
+
+
+sub exists_baginfo_key {
+    my ($self, $searchkey) =@_;
+    return (defined  $self->_find_baginfo_idx($searchkey));
+}
+
+###############################################
+
+sub _replace_baginfo_by_first_match {
+    my ($self, $searchkey, $newvalue) = @_;
+    my $idx = $self->_find_baginfo_idx( $searchkey);
+    if (defined $idx) {
+        $self->{bag_info}[$idx] = {$searchkey => $newvalue};
+        return $idx;
+    }
+    return;
+}
+
+###############################################
+
+
+sub append_baginfo_by_key {
+    my ($self, $searchkey, $newvalue) = @_;
+    if (defined $searchkey) {
+        if ($searchkey =~ m/:/) { croak "key should not contain a colon! (searchkey='$searchkey')"; }
+        if ($self->is_baginfo_key_reserved_as_uniq($searchkey)) {
+            if (defined $self->get_baginfo_values_by_key($searchkey)) {
+                # hmm, search key is mrked as uniq and still exists
+                return;
+            }
+        }
+        push @{$self->{bag_info}}, {$searchkey => $newvalue};
+    }
+    return 1;
+}
+
+###############################################
+
+
+sub add_or_replace_baginfo_by_key {
+    my ($self, $searchkey, $newvalue) = @_;
+    if (defined $searchkey) {
+        if ($searchkey =~ m/:/) { croak "key should not contain a colon! (searchkey='$searchkey')"; }
+        if (defined $self->{bag_info}) {
+            my $idx = $self->_replace_baginfo_by_first_match( $searchkey, $newvalue);
+            if (defined $idx) { return $idx;}
+        }
+        $self->append_baginfo_by_key( $searchkey, $newvalue );
+        return -1;
+    }
+}
+
+###############################################
+
+
+has 'forced_fixity_algorithm' => (
+    is   => 'ro',
+    lazy => 1,
+    builder  => '_build_forced_fixity_algorithm',
+);
+
+###############################################
+
+
+has 'manifest_files' => (
+    is => 'ro',
+    lazy => 1,
+    builder => '_build_manifest_files',
+);
+
+###############################################
+
+
+has 'tagmanifest_files' => (
+    is => 'ro',
+    lazy => 1,
+    builder => '_build_tagmanifest_files',
+);
+
+###############################################
+
+
+has 'payload_files' => ( # relatively to bagit base
+    is => 'ro',
+    lazy => 1,
+    builder => '_build_payload_files',
+);
+
+###############################################
+
+
+has 'non_payload_files' => (
+    is=>'ro',
+    lazy => 1,
+    builder => '_build_non_payload_files',
+);
+
+###############################################
+
+
+has 'plugins' => (
+    is=>'rw',
+    #isa=>'HashRef',
+);
+
+###############################################
+
+
+
+has 'manifests' => (
+    is      => 'rw',
+    lazy    => 1,
+    builder => '_build_manifests'
+    #isa=>'HashRef',
+);
+
+###############################################
+
+
+
+has 'algos' => (
+    is=>'rw',
+    #isa=>'HashRef',
+);
+
+###############################################
+
 sub _build_bag_path_arr {
     my ($self) = @_;
     my @split_path = File::Spec->splitdir($self->bag_path);
@@ -271,20 +479,11 @@ sub _build_checksum_algos {
     return $checksums;
 }
 
-sub _build_bag_checksum {
-  my($self) =@_;
-  my $bagit = $self->{'bag_path'};
-  open(my $SRCFILE, "<:raw",  $bagit."/manifest-md5.txt");
-  my $srchex=Digest::MD5->new->addfile($SRCFILE)->hexdigest;
-  close($SRCFILE);
-  return $srchex;
-}
-
 sub _build_manifest_files {
   my($self) = @_;
   my @manifest_files;
   foreach my $algo (@{$self->checksum_algos}) {
-    my $manifest_file = $self->metadata_path."/manifest-$algo.txt";
+    my $manifest_file = File::Spec->catdir($self->metadata_path, "manifest-$algo.txt");
     if (-f $manifest_file) {
       push @manifest_files, $manifest_file;
     }
@@ -296,7 +495,7 @@ sub _build_tagmanifest_files {
   my ($self) = @_;
   my @tagmanifest_files;
   foreach my $algo (@{$self->checksum_algos}) {
-    my $tagmanifest_file = $self->metadata_path."/tagmanifest-$algo.txt";
+    my $tagmanifest_file = File::Spec->catdir($self->metadata_path,"tagmanifest-$algo.txt");
     if (-f $tagmanifest_file) {
       push @tagmanifest_files, $tagmanifest_file;
     }
@@ -307,7 +506,6 @@ sub _build_tagmanifest_files {
 sub _build_payload_files{
   my($self) = @_;
   my $payload_dir = $self->payload_path;
-  my $payload_reldir = $self->rel_payload_path;
   my @payload=();
   File::Find::find( sub{
     $File::Find::name = decode ('utf8', $File::Find::name);
@@ -333,8 +531,8 @@ sub _build_payload_files{
 sub __build_read_bagit_txt {
     my($self) = @_;
     my $bagit = $self->metadata_path;
-    my $file = join("/", $bagit, "bagit.txt");
-    open(my $BAGIT, "<", $file) or die("Cannot read '$file': $!");
+    my $file = File::Spec->catdir($bagit, "bagit.txt");
+    open(my $BAGIT, "<", $file) or croak("Cannot read '$file': $!");
     my $version_string = <$BAGIT>;
     my $encoding_string = <$BAGIT>;
     close($BAGIT);
@@ -351,21 +549,21 @@ sub __build_read_bagit_txt {
 sub _build_bag_version {
     my($self) = @_;
     my ($version_string, $encoding_string, $file) = $self->__build_read_bagit_txt();
-    die "Version line missed in '$file" unless defined $version_string;
+    croak "Version line missed in '$file" unless defined $version_string;
     if ($version_string =~ /^BagIt-Version: ([01]\.[0-9]+)$/) {
         return $1
     } else {
         $version_string =~ s/\r/<CR>/;
         $version_string =~ s/^\N{U+FEFF}/<BOM>/;
-        die "Version string '$version_string' of '$file' is incorrect";
+        croak "Version string '$version_string' of '$file' is incorrect";
     };
 }
 
 sub _build_bag_encoding {
     my($self) = @_;
     my ($version_string, $encoding_string, $file) = $self->__build_read_bagit_txt();
-    die "Encoding line missed in '$file" unless defined $encoding_string;
-    die "Encoding '$encoding_string' of '$file' not supported by current Archive::BagIt module!" unless ($encoding_string !~ m/^UTF-8$/);
+    croak "Encoding line missed in '$file" unless defined $encoding_string;
+    croak "Encoding '$encoding_string' of '$file' not supported by current Archive::BagIt module!" unless ($encoding_string !~ m/^UTF-8$/);
     return $encoding_string;
 }
 
@@ -436,9 +634,9 @@ sub _parse_bag_info { # parses a bag-info textblob
 sub _build_bag_info {
     my ($self) = @_;
     my $bagit = $self->metadata_path;
-    my $file = join("/", $bagit, "bag-info.txt");
+    my $file = File::Spec->catdir($bagit, "bag-info.txt");
     if (-e $file) {
-        open(my $BAGINFO, "<", $file) or di("Cannot read $file: $!");
+        open(my $BAGINFO, "<", $file) or croak("Cannot read $file: $!");
         my @lines;
         foreach my $line (<$BAGINFO>) {
             push @lines, $line;
@@ -453,9 +651,7 @@ sub _build_bag_info {
 
 sub _build_non_payload_files {
   my($self) = @_;
-
   my @non_payload = ();
-
   File::Find::find( sub{
     $File::Find::name = decode('utf8', $File::Find::name);
     $_=decode ('utf8', $_);
@@ -488,6 +684,8 @@ sub _build_forced_fixity_algorithm {
     }
 }
 
+###############################################
+
 
 sub load_plugins {
     my ($self, @plugins) = @_;
@@ -498,15 +696,31 @@ sub load_plugins {
 
     return if @plugins == 0;
     foreach my $plugin (@plugins) {
-        load_class ($plugin) or die ("Can't load $plugin");
+        load_class ($plugin) or croak ("Can't load $plugin");
         $plugin->new({bagit => $self});
     }
 
     return 1;
 }
 
+###############################################
 
 
+sub load {
+    my ($self) = @_;
+    # call trigger
+    $self->bag_path;
+    $self->bag_version;
+    $self->bag_encoding;
+    $self->bag_info;
+    $self->payload_path;
+    $self->manifest_files;
+    $self->checksum_algos;
+    $self->tagmanifest_files;
+    return 1;
+}
+
+###############################################
 
 
 sub verify_bag {
@@ -516,21 +730,21 @@ sub verify_bag {
     my $bagit = $self->bag_path;
     my $version = $self->bag_version(); # to call trigger
     my $encoding = $self->bag_encoding(); # to call trigger
-    my $baginfo = $self->bag_info(); #to call trigger
+    my $baginfo = $self->verify_baginfo(); #to call trigger
     my $forced_fixity_alg = $self->forced_fixity_algorithm()->name();
-    my $fetch_file = $self->metadata_path."/fetch.txt";
-    my $manifest_file = $self->metadata_path."/manifest-$forced_fixity_alg.txt"; # FIXME: use plugin instead
+    my $fetch_file = File::Spec->catdir($self->metadata_path, "fetch.txt");
+    my $manifest_file = File::Spec->catdir($self->metadata_path, "manifest-$forced_fixity_alg.txt");
     my $payload_dir   = $self->payload_path;
     my $return_all_errors = $opts->{return_all_errors};
 
     if (-f $fetch_file) {
-        die("Fetching via file '$fetch_file' is not supported by current Archive::BagIt implementation")
+        croak("Fetching via file '$fetch_file' is not supported by current Archive::BagIt implementation")
     }
-    die("Manifest '$manifest_file' is not a regular file or does not exist for given bagit version '$version'") unless -f ($manifest_file);
-    die("Payload-directory '$payload_dir' is not a directory or does not exist") unless -d ($payload_dir);
+    croak("Manifest '$manifest_file' is not a regular file or does not exist for given bagit version '$version'") unless -f ($manifest_file);
+    croak("Payload-directory '$payload_dir' is not a directory or does not exist") unless -d ($payload_dir);
 
     unless ($version > .95) {
-        die ("Bag Version $version is unsupported");
+        croak ("Bag Version $version is unsupported");
     }
 
     # check forced fixity
@@ -538,17 +752,17 @@ sub verify_bag {
     my @errors;
     foreach my $algorithm ( keys %{ $self->manifests }) {
         my $res = $self->manifests->{$algorithm}->verify_manifest($self->payload_files, $return_all_errors);
-        if (defined $res && $res ne 1) { push @errors, $res; }
+        if ((defined $res) && ($res ne "1")) { push @errors, $res; }
     }
     foreach my $algorithm ( keys %{ $self->manifests }) {
         my $res = $self->manifests->{$algorithm}->verify_tagmanifest($self->non_payload_files, $return_all_errors);
-        if (defined $res && $res ne 1) { push @errors, $res; }
+        if ((defined $res) && ($res ne "1")) { push @errors, $res; }
     }
     push @{$self->{errors}}, @errors;
     my $err = $self->errors();
     my @err =  @{ $err };
     if (scalar( @err ) > 0) {
-        die join("\n","bag verify for bagit version '$version' failed with invalid files.", @err);
+        croak join("\n","bag verify for bagit version '$version' failed with invalid files.", @err);
     }
     return 1;
 }
@@ -560,7 +774,7 @@ sub calc_payload_oxum {
     my $octets=0;
     my $streamcount = scalar @payload;
     foreach my $local_name (@payload) {# local_name is relative to bagit base
-        my $file = $self->bag_path()."/$local_name";
+        my $file = File::Spec->catdir($self->bag_path(), $local_name);
         my $sb = stat($file);
         $octets += $sb->size;
     }
@@ -582,7 +796,8 @@ sub calc_bagsize {
 sub create_bagit {
     my($self) = @_;
     my $metadata_path = $self->metadata_path();
-    open(my $BAGIT, ">", "$metadata_path/bagit.txt") or die("Can't open $metadata_path/bagit.txt for writing: $!");
+    my $bagit_path = File::Spec->catdir( $metadata_path, "bagit.txt");
+    open(my $BAGIT, ">", $bagit_path) or croak("Can't open $bagit_path for writing: $!");
     print($BAGIT "BagIt-Version: 1.0\nTag-File-Character-Encoding: UTF-8");
     close($BAGIT);
     return 1;
@@ -591,14 +806,15 @@ sub create_bagit {
 
 sub create_baginfo {
     my($self) = @_; # because bag-info.txt allows multiple key-value-entries, hash is replaced
-    $self->_add_or_replace_bag_info('Bagging-Date', POSIX::strftime("%F", gmtime(time)));
-    $self->_add_or_replace_bag_info('Bag-Software-Agent', 'Archive::BagIt <https://metacpan.org/pod/Archive::BagIt>');
+    $self->add_or_replace_baginfo_by_key('Bagging-Date', POSIX::strftime("%Y-%m-%D", gmtime(time)));
+    $self->add_or_replace_baginfo_by_key('Bag-Software-Agent', 'Archive::BagIt <https://metacpan.org/pod/Archive::BagIt>');
     my ($octets, $streams) = $self->calc_payload_oxum();
-    $self->_add_or_replace_bag_info('Payload-Oxum', "$octets.$streams");
-    $self->_add_or_replace_bag_info('Bag-Size', $self->calc_bagsize());
+    $self->add_or_replace_baginfo_by_key('Payload-Oxum', "$octets.$streams");
+    $self->add_or_replace_baginfo_by_key('Bag-Size', $self->calc_bagsize());
     # The RFC does not allow reordering:
     my $metadata_path = $self->metadata_path();
-    open(my $BAGINFO, ">", "$metadata_path/bag-info.txt") or die("Can't open $metadata_path/bag-info.txt for writing: $!");
+    my $bag_info_path = File::Spec->catdir( $metadata_path, "bag-info.txt");
+    open(my $BAGINFO, ">", $bag_info_path) or croak("Can't open $bag_info_path for writing: $!");
     foreach my $entry (@{ $self->bag_info() }) {
         my %tmp = %{ $entry };
         my ($key, $value) = each %tmp;
@@ -632,9 +848,9 @@ sub store {
 sub init_metadata {
     my ($class, $bag_path) = @_;
     $bag_path =~ s#/$##; # replace trailing slash
-    unless ( -d $bag_path) { die ( "source bag directory '$bag_path' doesn't exist"); }
+    unless ( -d $bag_path) { croak ( "source bag directory '$bag_path' doesn't exist"); }
     my $self = $class->new(bag_path=>$bag_path);
-    warn "no payload path" if ! -d $self->payload_path;
+    carp "no payload path" if ! -d $self->payload_path;
     unless ( -d $self->payload_path) {
         rename ($bag_path, $bag_path.".tmp");
         mkdir  ($bag_path);
@@ -662,7 +878,7 @@ sub make_bag {
   my ($class, $bag_path) = @_;
   my $isa = ref $class;
   if ($isa eq "Archive::BagIt::Base") { # not a class, but an object!
-    die "make_bag() only a class subroutine, not useable with objects. Try store() instead!\n";
+    croak "make_bag() only a class subroutine, not useable with objects. Try store() instead!\n";
   }
   my $self = $class->init_metadata($bag_path);
   return $self;
@@ -681,11 +897,11 @@ __END__
 
 =head1 NAME
 
-Archive::BagIt::Base
+Archive::BagIt::Base - The common base for Archive::BagIt. This is the module for experts. ;)
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 SYNOPSIS
 
@@ -719,13 +935,23 @@ You only need to know the following methods first:
     my $bag4 = Archive::BagIt::Base->new($bag_dir);
     my $is_valid2 = $bag4->verify_bag( {report_all_errors => 1} );
 
+=head2 read a BagIt-dir, change something, store
+
+Because all methods operate lazy, you should ensure to parse parts of the bag *BEFORE* you modify it.
+Otherwise it will be overwritten!
+
+    use Archive::BagIt::Base;
+    my $bag5 = Archive::BagIt::Base->new($bag_dir); # lazy, nothing happened
+    $bag5->load(); # this updates the object representation by parsing the given $bag_dir
+    $bag5->store(); # this writes the bag new
+
 =head1 NAME
 
-Archive::BagIt::Base
+Archive::BagIt::Base - The common base for Archive::BagIt. This is the module for experts. ;)
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 NAME
 
@@ -792,22 +1018,59 @@ See L<https://datatracker.ietf.org/doc/rfc8493/?include_text=1> for details.
 
 =over
 
-=item Add support for non-Unix based filesystems
-
 =item enhanced testsuite
 
-=item improved plugin mechanism
-
 =item reduce complexity
 
 =item use modern perl code
 
 =item add code to easily update outdated Bags to v1.0
 
-=item add more ecamples in documentation
-
 =back
 
+=head1 FAQ
+
+=head2 How to access the manifest-entries directly?
+
+Try this:
+
+   foreach my $algorithm ( keys %{ $self->manifests }) {
+       my $entries_ref = $self->manifests->{$algorithm}->manifest_entries();
+       # $entries_ref returns a hashref of form:
+       # $entries_ref->{$algorithm}->{$file} = $digest;
+   }
+
+Similar for tagmanifests
+
+=head2 How fast is C<Archive::BagIt::Fast>?
+
+It depends. On my system with SSD and a 38MB bag with 48 payload files the results for C<verify_bag()> are:
+
+                  Rate BaseParallel FastParallel         Base         Fast
+   BaseParallel 2.46/s           --          -2%         -52%         -57%
+   FastParallel 2.52/s           2%           --         -51%         -56%
+   Base         5.10/s         107%         102%           --         -10%
+   Fast         5.69/s         131%         125%          11%           --
+
+On network filesystem (CIFS, 1GB) with same Bag:
+
+                  Rate FastParallel         Fast BaseParallel         Base
+   FastParallel 1.97/s           --         -10%         -15%         -20%
+   Fast         2.20/s          12%           --          -6%         -11%
+   BaseParallel 2.33/s          18%           6%           --          -6%
+   Base         2.48/s          26%          13%           6%           --
+
+But you should measure which variant is best for you. In general the default C<Archive::BagIt::Base> is fast enough.
+
+=head2 How to update an old bag of version v0.97 to v1.0?
+
+You could try this:
+
+   use Archive::BagIt::Base;
+   my $bag=Archive::BagIt::Base->new( $my_old_bag_filepath );
+   $bag->load();
+   $bag->store();
+
 =head1 METHODS
 
 =head2 Constructor
@@ -842,16 +1105,143 @@ The arguments are:
 
 =back
 
+The bag object will use $bag_dir, BUT an existing $bag_dir is not read. If you use C<store()> an existing bag will be overwritten!
+
+See C<load()> if you want to parse/modify an existing bag.
+
+=head2 has_parallel()
+
+to check if parallelization is possible.
+
+=head2 bag_path([$new_value])
+
+Getter/setter for bag path
+
+=head2 metadata_path()
+
+Getter for metadata path
+
+=head2 payload_path()
+
+Getter for payload path
+
+=head2 checksum_algos()
+
+Getter for registered Checksums
+
+=head2 bag_version()
+
+Getter for bag version
+
+=head2 bag_encoding()
+
+Getter for bag encoding.
+
+HINT: the current version of Archive::BagIt::Base only supports UTF-8, but the method could return other values depending on given Bags.
+
+=head2 bag_info([$new_value])
+
+Getter/Setter for bag info. Expects/returns an array of HashRefs implementing simple key-value pairs.
+
+HINT: RFC8493 does not allow *reordering* of entries!
+
+=head2 errors()
+
+Getter to return collected errors after a C<verify_bag()> call with Option C<report_all_errors>
+
+=head2 digest_callback()
+
+This method could be reimplemented by derived classes to handle fixity checks in own way. The
+getter returns an anonymous function with following interface:
+
+   my $digest = $self->digest_callback;
+   &$digest( $digestobject, $filename);
+
+This anonymous function MUST use the C<get_hash_string()> function of the C<Archive::BagIt::Role::Algorithm> role,
+which is implemented by each C<Archive::BagIt::Plugin::Algorithm::XXXX> module.
+
+See C<Archive::BagIt::Fast> for details.
+
+=head2 get_baginfo_values_by_key($searchkey)
+
+Returns all values which match $searchkey, undef otherwise
+
+=head2 is_baginfo_key_reserved_as_uniq($searchkey)
+
+returns true if key is reserved and should be uniq
+
+=head2 is_baginfo_key_reserved( $searchkey )
+
+returns true if key is reserved
+
+=head2 verify_baginfo()
+
+checks baginfo-keys, returns true if all fine, otherwise returns undef and the message is pushed to C<errors()>.
+
+=head2 delete_baginfo_by_key( $searchkey )
+
+deletes an entry of given $searchkey if exists
+
+=head2 exists_baginfo_key( $searchkey )
+
+returns true if a given $searchkey exists
+
+=head2 append_baginfo_by_key($searchkey, $newvalue)
+
+Appends a key value pair to bag_info.
+
+HINT: check return code if append was successful, because some keys needs to be uniq.
+
+=head2 add_or_replace_baginfo_by_key($searchkey, $newvalue)
+
+It replaces the first entry with $newvalue if $searchkey exists, otherwise it appends.
+
+=head2 forced_fixity_algorithm()
+
+Getter to return the forced fixity algorithm depending on BagIt version
+
+=head2 manifest_files()
+
+Getter to find all manifest-files
+
+=head2 tagmanifest_files()
+
+Getter to find all tagmanifest-files
+
+=head2 payload_files()
+
+Getter to find all payload-files
+
+=head2 non_payload_files()
+
+Getter to find all non payload-files
+
+=head2 plugins()
+
+Getter/setter to algorithm plugins
+
+=head2 manifests()
+
+Getter/Setter to all manifests (objects)
+
+=head2 algos()
+
+Getter/Setter to all registered Algorithms
+
 =head2 load_plugins
 
 As default SHA512 and MD5 will be loaded and therefore used. If you want to create a bag only with one or a specific
 checksum-algorithm, you could use this method to (re-)register it. It expects list of strings with namespace of type:
 Archive::BagIt::Plugin::Algorithm::XXX where XXX is your chosen fixity algorithm.
 
+=head2 load()
+
+Triggers loading of an existing bag
+
 =head2 verify_bag($opts)
 
 A method to verify a bag deeply. If C<$opts> is set with C<{return_all_errors}> all fixity errors are reported.
-The default ist to die with error message if any error is detected.
+The default ist to croak with error message if any error is detected.
 
 HINT: You might also want to check Archive::BagIt::Fast to see a more direct way of accessing files (and thus faster).
 
@@ -869,7 +1259,7 @@ creates a bagit.txt file
 
 =head2 create_baginfo()
 
-creates a bag-info.tx
+creates a bag-info.txt file
 
 =head2 store()
 
@@ -894,11 +1284,6 @@ The latest version of this module is available from the Comprehensive Perl
 Archive Network (CPAN). Visit L<http://www.perl.com/CPAN/> to find a CPAN
 site near you, or see L<https://metacpan.org/module/Archive::BagIt/>.
 
-=head1 SOURCE
-
-The development version is on github at L<https://github.com/Archive-BagIt>
-and may be cloned from L<git://github.com/Archive-BagIt.git>
-
 =head1 BUGS AND LIMITATIONS
 
 You can make new bug reports, and view existing ones, through the
