@@ -1,18 +1,17 @@
-use strict;
-use warnings;
-
 package Archive::BagIt::DotBagIt;
 
-our $VERSION = '0.060'; # VERSION
-
+our $VERSION = '0.061'; # VERSION
+use strict;
+use warnings;
 use Sub::Quote;
+use Carp;
+use File::Spec;
 use Moo;
-
 extends "Archive::BagIt::Base";
 
 
 before BUILDARGS => sub {
-    warn "The module Archive::BagIt::DotBagIt is marked as deprecated and will be erased in next releases!";
+    carp "The module Archive::BagIt::DotBagIt is marked as deprecated and will be erased in next releases!";
 };
 
 has 'metadata_path' => (
@@ -24,7 +23,7 @@ has 'metadata_path' => (
 sub _build_metadata_path {
     my ($self) = @_;
     my $bag_path = $self->bag_path();
-    return "$bag_path/.bagit";
+    return File::Spec->catdir($bag_path, ".bagit");
 }
 
 has 'payload_path' => (
@@ -52,7 +51,7 @@ Archive::BagIt::DotBagIt
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 NAME
 
@@ -60,7 +59,7 @@ Archive::BagIt::DotBagIt
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 NAME
 
@@ -72,11 +71,6 @@ The latest version of this module is available from the Comprehensive Perl
 Archive Network (CPAN). Visit L<http://www.perl.com/CPAN/> to find a CPAN
 site near you, or see L<https://metacpan.org/module/Archive::BagIt/>.
 
-=head1 SOURCE
-
-The development version is on github at L<https://github.com/Archive-BagIt>
-and may be cloned from L<git://github.com/Archive-BagIt.git>
-
 =head1 BUGS AND LIMITATIONS
 
 You can make new bug reports, and view existing ones, through the
