@@ -1,11 +1,9 @@
+#ABSTRACT: The MD5 algorithm plugin (default for v0.97)
+package Archive::BagIt::Plugin::Algorithm::MD5;
 use strict;
 use warnings;
-
-#ABSTRACT: The default MD5 algorithm plugin
-
-package Archive::BagIt::Plugin::Algorithm::MD5;
-
 use Moo;
+use Carp;
 use namespace::autoclean;
 
 with 'Archive::BagIt::Role::Algorithm';
@@ -30,7 +28,7 @@ has '_digest' => (
 
 sub _build_digest_md5 {
     my ($self) = @_;
-    my $digest_md5 = new Digest::MD5;
+    my $digest_md5 = Digest::MD5->new();
     return $digest_md5;
 }
 
@@ -52,10 +50,10 @@ sub get_hash_string {
 
 sub verify_file {
     my ($self, $filename) = @_;
-    open(my $fh, '<', $filename) || die ("Can't open '$filename', $!");
+    open(my $fh, '<', $filename) || croak ("Can't open '$filename', $!");
     binmode($fh);
     my $digest = $self->get_hash_string($fh);
-    close $fh || die("could not close file '$filename', $!");
+    close $fh || croak("could not close file '$filename', $!");
     return $digest;
 }
 __PACKAGE__->meta->make_immutable;
@@ -69,19 +67,19 @@ __END__
 
 =head1 NAME
 
-Archive::BagIt::Plugin::Algorithm::MD5 - The default MD5 algorithm plugin
+Archive::BagIt::Plugin::Algorithm::MD5 - The MD5 algorithm plugin (default for v0.97)
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 NAME
 
-Archive::BagIt::Plugin::Algorithm::MD5 - The default MD5 algorithm plugin
+Archive::BagIt::Plugin::Algorithm::MD5 - The MD5 algorithm plugin (default for v0.97)
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 AVAILABILITY
 
@@ -89,11 +87,6 @@ The latest version of this module is available from the Comprehensive Perl
 Archive Network (CPAN). Visit L<http://www.perl.com/CPAN/> to find a CPAN
 site near you, or see L<https://metacpan.org/module/Archive::BagIt/>.
 
-=head1 SOURCE
-
-The development version is on github at L<https://github.com/Archive-BagIt>
-and may be cloned from L<git://github.com/Archive-BagIt.git>
-
 =head1 BUGS AND LIMITATIONS
 
 You can make new bug reports, and view existing ones, through the
