@@ -1,13 +1,10 @@
+#ABSTRACT: The default SHA algorithms plugin (default for v1.0)
+package Archive::BagIt::Plugin::Algorithm::SHA512;
 use strict;
 use warnings;
-
-#ABSTRACT: The default SHA algorithms plugin
-
-package Archive::BagIt::Plugin::Algorithm::SHA512;
-
+use Carp;
 use Moo;
 use namespace::autoclean;
-
 with 'Archive::BagIt::Role::Algorithm';
 
 has '+plugin_name' => (
@@ -51,10 +48,10 @@ sub get_hash_string {
 
 sub verify_file {
     my ($self, $filename) = @_;
-    open(my $fh, '<', $filename) || die ("Can't open '$filename', $!");
+    open(my $fh, '<', $filename) || croak ("Can't open '$filename', $!");
     binmode($fh);
     my $digest = $self->get_hash_string($fh);
-    close $fh || die("could not close file '$filename', $!");
+    close $fh || croak("could not close file '$filename', $!");
     return $digest;
 }
 __PACKAGE__->meta->make_immutable;
@@ -68,19 +65,19 @@ __END__
 
 =head1 NAME
 
-Archive::BagIt::Plugin::Algorithm::SHA512 - The default SHA algorithms plugin
+Archive::BagIt::Plugin::Algorithm::SHA512 - The default SHA algorithms plugin (default for v1.0)
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 NAME
 
-Archive::BagIt::Plugin::Algorithm::SHA512 - The default SHA algorithms plugin
+Archive::BagIt::Plugin::Algorithm::SHA512 - The default SHA algorithms plugin (default for v1.0)
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 AVAILABILITY
 
@@ -88,11 +85,6 @@ The latest version of this module is available from the Comprehensive Perl
 Archive Network (CPAN). Visit L<http://www.perl.com/CPAN/> to find a CPAN
 site near you, or see L<https://metacpan.org/module/Archive::BagIt/>.
 
-=head1 SOURCE
-
-The development version is on github at L<https://github.com/Archive-BagIt>
-and may be cloned from L<git://github.com/Archive-BagIt.git>
-
 =head1 BUGS AND LIMITATIONS
 
 You can make new bug reports, and view existing ones, through the
