@@ -1,12 +1,13 @@
+#ABSTRACT: A role that handles all manifest files for a specific Algorithm
+package Archive::BagIt::Role::Manifest;
 use strict;
 use warnings;
-package Archive::BagIt::Role::Manifest;
-
+use namespace::autoclean;
+use Carp;
+use File::Spec;
 use Moo::Role;
 with 'Archive::BagIt::Role::Plugin';
 
-use namespace::autoclean;
-
 has 'algorithm' => (
     is => 'rw',
     #isa=>'HashRef',
@@ -21,7 +22,7 @@ has 'manifest_file' => (
 sub _build_manifest_file {
     my $self = shift;
     my $algorithm = $self->algorithm()->name;
-    my $file = $self->bagit->metadata_path."/manifest-$algorithm.txt";
+    my $file = File::Spec->catdir($self->bagit->metadata_path, "manifest-$algorithm.txt");
     if (-f $file) {
         return $file;
     }
@@ -38,7 +39,7 @@ has 'tagmanifest_file' => (
 sub _build_tagmanifest_file {
     my $self = shift;
     my $algorithm = $self->algorithm()->name;
-    my $file = $self->bagit->metadata_path."/tagmanifest-$algorithm.txt";
+    my $file = File::Spec->catdir( $self->bagit->metadata_path, "tagmanifest-$algorithm.txt");
     if (-f $file) {
         return $file;
     }
@@ -78,14 +79,13 @@ has 'tagmanifest_entries' => (
 sub __build_xxxmanifest_entries {
     my ($self, $xxmanifest_file) = @_;
     my $xxmanifest_entries = {};
-    my $bag_path = $self->bagit->bag_path();
-    die("Cannot open $xxmanifest_file: $!") unless (open(my $XXMANIFEST, "<:encoding(utf8)", $xxmanifest_file));
-    my $algo = $self->algorithm()->name;
+    open(my $XXMANIFEST, "<:encoding(utf8)", $xxmanifest_file) or croak("Cannot open $xxmanifest_file: $!");
+    my $algorithm = $self->algorithm()->name;
     while (my $line = <$XXMANIFEST>) {
         chomp($line);
         my ($digest, $file) = split(/\s+/, $line, 2);
         next unless ((defined $digest) && (defined $file)); # empty lines!
-        $xxmanifest_entries->{$algo}->{$file} = $digest;
+        $xxmanifest_entries->{$algorithm}->{$file} = $digest;
     }
     close($XXMANIFEST);
     return $xxmanifest_entries;
@@ -120,7 +120,6 @@ sub normalize_payload_filepath {
     return $filename;
 }
 
-
 sub check_if_payload_filepath_violates{
     my $local_name = shift;
     # HINT: there is no guarantuee *not* to escape!
@@ -138,17 +137,18 @@ sub check_if_payload_filepath_violates{
     ;
 }
 
-sub __calc_digest {
-    my ($digestobj, $filename) = @_;
-    open(my $fh, '<', $filename) || die("Can't open '$filename', $!");
-    binmode($fh);
-    my $digest = $digestobj->get_hash_string($fh);
-    close $fh || die("could not close file '$filename', $!");
-    return $digest;
+sub _fill_digest_hashref {
+    my ($self, $bagit, $digestobj, $localname) = @_;
+    my $digest_hashref;
+    my $fullname = File::Spec->catdir($bagit, $localname);
+    my $calc_digest = $self->bagit->digest_callback();
+    $digest_hashref->{calculated_digest} = &$calc_digest($digestobj, $fullname);
+    $digest_hashref->{local_name} = $localname;
+    $digest_hashref->{full_name} = $fullname;
+    return $digest_hashref;
 }
 
 
-
 # calc digest
 # expects digestobj, expected_ref, array_ref of filenames
 # returns arrayref of hashes where each entry has
@@ -164,60 +164,52 @@ sub calc_digests {
         my $err;
         ($is_parallelizeable, $err) = Class::Load::try_load_class("Parallel::Iterator");
         if (!$is_parallelizeable) {
-            warn "Class 'Parallel::Iterator' could not be loaded…, $err\n";
+            carp "Class 'Parallel::Iterator' could not be loaded…, $err\n";
             $self->{parallel} = undef;
         }
     }
     if ($is_parallelizeable) {
         my $class = Class::Load::load_class("Parallel::Iterator");
-        $class->import( qw(iterate_as_array));
-        @digest_hashes = iterate_as_array(
-            sub {
-                my ($idx, $localname) = @_;
-                my $fullname = $bagit ."/". $localname;
-                my $tmp;
-                $tmp->{calculated_digest} = __calc_digest($digestobj, $fullname);
-                $tmp->{local_name} = $localname;
-                $tmp->{full_name} = $fullname;
-                $tmp;
-            }, $filenames_ref);
+        $class->import( qw(iterate));
+        my $worker = sub {
+            my ($idx, $localname) = @_;
+            return $self->_fill_digest_hashref($bagit, $digestobj, $localname);
+        };
+        my $iter = iterate( { adaptive=>2 }, $worker, $filenames_ref);
+        while ( my ($index, $value) = $iter->() ) {
+            $digest_hashes[$index] = $value;
+        }
     }
     else { # fallback to serial processing
         @digest_hashes = map {
-            my $localname = $_;
-            my $fullname = $bagit ."/". $localname;
-            my $tmp;
-            $tmp->{calculated_digest} = __calc_digest($digestobj, $fullname);
-            $tmp->{local_name} = $localname;
-            $tmp->{full_name} = $fullname;
-            $tmp;
+            $self->_fill_digest_hashref($bagit, $digestobj, $_);
         } @{$filenames_ref};
     }
     return \@digest_hashes;
 }
 
-
-
 sub _verify_XXX_manifests {
-    my ($self, $xxprefix, $xxmanifest_entries, $files, $return_all_errors) =@_;
+    my ($self, $xxprefix, $xxmanifest_entries, $files_ref, $return_all_errors) = @_;
     # Read the manifest file
-    my @files = grep {$_ !~ m/$xxprefix-[0-9a-zA-Z]+\.txt/}@{ $files };
+    my @files = @{ $files_ref };
     my @invalid_messages;
     my $bagit = $self->bagit->bag_path;
-    my $version = $self->bagit->bag_version();
-    my $alg = $self->algorithm()->name;
+    my $algorithm = $self->algorithm()->name;
     my $subref_invalid_report_or_die = sub {
         my $message = shift;
         if (defined $return_all_errors) {
             push @invalid_messages, $message;
-        } else {
-            die($message);
+        }
+        else {
+            croak($message);
         }
         return;
     };
+    # Test readability
     foreach my $local_name (@files) {
         # local_name is relative to bagit base
-        unless (-r $bagit."/".$local_name) {
+        my $filepath = File::Spec->catdir($bagit, $local_name);
+        unless (-r $filepath) {
             &$subref_invalid_report_or_die(
                 "cannot read $local_name (bag-path:$bagit)",
             );
@@ -225,59 +217,62 @@ sub _verify_XXX_manifests {
     }
     # Evaluate each file against the manifest
 
-        my $digestobj = $self->algorithm();
-        my $local_xxfilename = "${xxprefix}-${alg}.txt";
-        my $xxfilename = "${bagit}/$local_xxfilename";
+    my $local_xxfilename = "${xxprefix}-${algorithm}.txt";
 
-        # first check if each file from payload exists in manifest_entries for given alg
-        foreach my $local_name (@files) {
-            my $normalized_local_name = normalize_payload_filepath($local_name);
-            # local_name is relative to bagit base
-            unless (exists $xxmanifest_entries->{$alg}->{$normalized_local_name}) { # localname as value should exist!
+    # first check if each file from payload exists in manifest_entries for given alg
+    foreach my $local_name (@files) {
+        my $normalized_local_name = normalize_payload_filepath($local_name);
+        # local_name is relative to bagit base
+        unless (exists $xxmanifest_entries->{$algorithm}->{$normalized_local_name}) { # localname as value should exist!
+            &$subref_invalid_report_or_die(
+                "file '$local_name' (normalized='$normalized_local_name') found, which is not in '$local_xxfilename' (bag-path:'$bagit')!"
+                    #."DEBUG: \n".join("\n", keys %{$xxmanifest_entries->{$algorithm}})
+            );
+        }
+    }
+    # second check if each file from manifest_entries for given alg exists in payload
+    foreach my $local_mf_entry_path (keys %{$xxmanifest_entries->{$algorithm}}) {
+        if ( # to avoid escapes via manifest-files
+            check_if_payload_filepath_violates($local_mf_entry_path)
+        ) {
+            &$subref_invalid_report_or_die("file '$local_mf_entry_path' not allowed in '$local_xxfilename' (bag-path:'$bagit'")
+        }
+        else {
+            unless (List::Util::any {normalize_payload_filepath($_) eq $local_mf_entry_path} @files) {
                 &$subref_invalid_report_or_die(
-                    "file '$local_name' (normalized='$normalized_local_name') found, which is not in '$local_xxfilename' (bag-path:'$bagit')!"
-                #"DEBUG: \n".join("\n", keys %{$xxmanifest_entries->{$alg}}),
+                    "file '$local_mf_entry_path' NOT found, but expected via '$local_xxfilename' (bag-path:'$bagit')!"
                 );
             }
         }
-        # second check if each file from manifest_entries for given alg exists in payload
-        foreach my $local_name (keys %{$xxmanifest_entries->{$alg}}) {
-            if ( # to avoid escapes via manifest-files
-                check_if_payload_filepath_violates( $local_name )
-            ) {
-                &$subref_invalid_report_or_die("file '$local_name' not allowed in '$local_xxfilename' (bag-path:'$bagit'")
-            }
-            else {
-                unless (List::Util::any {normalize_payload_filepath($_) eq $local_name} @files) {
-                    &$subref_invalid_report_or_die(
-                        "file '$local_name' NOT found, but expected via '$local_xxfilename' (bag-path:'$bagit')!"
-                    );
-                }
-            }
-        }
-        # all preconditions full filled, now calc all digests
-        my $digest_hashes_ref = $self->calc_digests($bagit, $digestobj, \@files);
-        # compare digests
-        if (defined $digest_hashes_ref && (ref $digest_hashes_ref eq 'ARRAY')) {
-            foreach my $digest_entry (@{$digest_hashes_ref}) {
-                my $normalized = normalize_payload_filepath($digest_entry->{local_name});
-                $digest_entry->{expected_digest} = $xxmanifest_entries->{$alg}->{$normalized};
-                if ($digest_entry->{calculated_digest} ne $digest_entry->{expected_digest}) {
-                    &$subref_invalid_report_or_die(
-                        sprintf("file '%s' (normalized='$normalized') invalid, digest (%s) calculated=%s, but expected=%s in file '%s'",
-                            $digest_entry->{local_name},
-                            $alg,
-                            $digest_entry->{calculated_digest},
-                            $digest_entry->{expected_digest},
-                            $xxfilename
-                        )
-                    );
-                }
+    }
+    # all preconditions full filled, now calc all digests
+    my $digestobj = $self->algorithm();
+    my $digest_hashes_ref = $self->calc_digests($bagit, $digestobj, \@files);
+    # compare digests
+    if (defined $digest_hashes_ref && (ref $digest_hashes_ref eq 'ARRAY')) {
+        foreach my $digest_entry (@{$digest_hashes_ref}) {
+            my $normalized = normalize_payload_filepath($digest_entry->{local_name});
+            $digest_entry->{expected_digest} = $xxmanifest_entries->{$algorithm}->{$normalized};
+            #use Data::Printer; p( $digest_entry); p( $local_xxfilename);p( $algorithm);p($normalized);
+            if (! defined $digest_entry->{expected_digest} ) { next; } # undef expected digests only occur if all preconditions fullfilled but return_all_errors was set, we should ignore it!
+            if ($digest_entry->{calculated_digest} ne $digest_entry->{expected_digest}) {
+                my $xxfilename = File::Spec->catdir($bagit, $local_xxfilename);
+                &$subref_invalid_report_or_die(
+                    sprintf("file '%s' (normalized='%s') invalid, digest (%s) calculated=%s, but expected=%s in file '%s'",
+                        $digest_entry->{local_name},
+                        $normalized,
+                        $algorithm,
+                        $digest_entry->{calculated_digest},
+                        $digest_entry->{expected_digest},
+                        $xxfilename
+                    )
+                );
             }
         }
+    }
 
-    if($return_all_errors && (scalar @invalid_messages > 0)) {
-        push @{ $self->bagit->{errors} },
+    if ($return_all_errors && (scalar @invalid_messages > 0)) {
+        push @{$self->bagit->{errors}},
             join("\n\t",
                 sort @invalid_messages
             );
@@ -286,72 +281,67 @@ sub _verify_XXX_manifests {
     return 1;
 }
 
+
 sub verify_manifest {
-    my ($self, $payload_files, $return_all_errors) = @_;
+    my ($self, $payload_files_ref, $return_all_errors) = @_;
     if ($self->manifest_file()) {
         return $self->_verify_XXX_manifests(
             "manifest",
             $self->manifest_entries(),
-            $payload_files,
+            $payload_files_ref,
             $return_all_errors
         );
     }
     return;
 }
 
+
 sub verify_tagmanifest {
-    my ($self, $non_payload_files, $return_all_errors) = @_;
+    my ($self, $non_payload_files_ref, $return_all_errors) = @_;
+    my @non_payload_files = grep {$_ !~ m#tagmanifest-[0-9a-zA-Z]+\.txt$#} @{ $non_payload_files_ref };
     if ($self->tagmanifest_file()) {
         return $self->_verify_XXX_manifests(
             "tagmanifest",
             $self->tagmanifest_entries(),
-            $non_payload_files,
+            \@non_payload_files,
             $return_all_errors
         );
     }
     return;
 }
 
+sub __create_xxmanifest {
+    my ($self, $prefix, $files_ref) = @_;
+    my $algo = $self->algorithm->name;
+    my $digestobj = $self->algorithm;
+    my $bagit = $self->bagit->bag_path;
+    my $manifest_file = File::Spec->catdir($self->bagit->metadata_path, "$prefix-${algo}.txt");
+    # Generate digests for all of the files under ./data
+    my $digest_hashes_ref = $self->calc_digests($bagit, $digestobj, $files_ref);
+    if (defined $digest_hashes_ref && (ref $digest_hashes_ref eq 'ARRAY')) {
+        open(my $fh, ">",$manifest_file) or croak("Cannot create $prefix-${algo}.txt: $!\n");
+        foreach my $digest_entry (@{$digest_hashes_ref}) {
+            my $normalized_file = normalize_payload_filepath($digest_entry->{local_name});
+            my $digest = $digest_entry->{calculated_digest};
+            print($fh "$digest  $normalized_file\n");
+        }
+        close($fh);
+    }
+    return 1;
+}
+
+
 sub create_manifest {
     my ($self) = @_;
-    my $algo = $self->algorithm->name;
-    my $manifest_file = $self->bagit->metadata_path."/manifest-${algo}.txt";
-    # Generate digests for all of the files under ./data
-    open(my $fh, ">",$manifest_file) or die("Cannot create manifest-${algo}.txt: $!\n");
-    foreach my $rel_payload_file (@{$self->bagit->payload_files}) {
-        #print "rel_payload_file: ".$rel_payload_file;
-        my $payload_file = File::Spec->catdir($self->bagit->bag_path, $rel_payload_file);
-        my $digest = $self->algorithm->verify_file( $payload_file );
-        my $normalized_payload_file = normalize_payload_filepath($rel_payload_file);
-        print($fh "$digest  $normalized_payload_file\n");
-        #print "lineout: $digest $filename\n";
-    }
-    close($fh);
+    $self->__create_xxmanifest('manifest', $self->bagit->payload_files);
     return 1;
-
 }
 
+
 sub create_tagmanifest {
     my ($self) = @_;
-    my $algo = $self->algorithm->name;
-    my $tagmanifest_file = $self->bagit->metadata_path . "/tagmanifest-${algo}.txt";
-    open(my $fh, ">", $tagmanifest_file) or die("Cannot create tagmanifest-${algo}.txt: $! \n");
-    foreach my $rel_nonpayload_file (@{$self->bagit->non_payload_files}) {
-        my $nonpayload_file = File::Spec->catdir($self->bagit->bag_path, $rel_nonpayload_file);
-        if ($rel_nonpayload_file =~ m/tagmanifest-.*\.txt$/) {
-            # Ignore, we can't take digest from ourselves
-        }
-        elsif (-f $nonpayload_file) {
-            # non-payload is all which is not payload, this allows user to define and handle own subdirs
-            my $digest = $self->algorithm->verify_file($nonpayload_file);
-            my $normalized_nonpayload_file = normalize_payload_filepath($rel_nonpayload_file);
-            print($fh "$digest  $normalized_nonpayload_file\n");
-        }
-        else {
-            die("A file or directory that doesn't match: $rel_nonpayload_file");
-        }
-    }
-    close($fh);
+    my @non_payload_files = grep {$_ !~ m#^tagmanifest-.*\.txt$#} @{ $self->bagit->non_payload_files };
+    $self->__create_xxmanifest('tagmanifest', \@non_payload_files);
     return 1;
 }
 
@@ -366,19 +356,19 @@ __END__
 
 =head1 NAME
 
-Archive::BagIt::Role::Manifest
+Archive::BagIt::Role::Manifest - A role that handles all manifest files for a specific Algorithm
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 NAME
 
-Archive::BagIt::Role::Manifest
+Archive::BagIt::Role::Manifest - A role that handles all manifest files for a specific Algorithm
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head2 calc_digests($bagit, $digestobj, $filenames_ref, $opts)
 
@@ -386,17 +376,28 @@ Method to calculate and return all digests for a a list of files using a Digest-
 
 If object ist build with option C<parallel>, the digests will be build in parallel.
 
+=head2 verify_manifest($payload_files, $return_all_errors)
+
+check fixities of payload files in both directions
+
+=head2 verify_tagmanifest($non_payload_files, $return_all_errors)
+
+check fixities of non-payload files in both directions
+
+=head2 create_manifest()
+
+creates a new manifest file for payload files
+
+=head2 create_tagmanifest()
+
+creates a new tagmanifest file for non payload files
+
 =head1 AVAILABILITY
 
 The latest version of this module is available from the Comprehensive Perl
 Archive Network (CPAN). Visit L<http://www.perl.com/CPAN/> to find a CPAN
 site near you, or see L<https://metacpan.org/module/Archive::BagIt/>.
 
-=head1 SOURCE
-
-The development version is on github at L<https://github.com/Archive-BagIt>
-and may be cloned from L<git://github.com/Archive-BagIt.git>
-
 =head1 BUGS AND LIMITATIONS
 
 You can make new bug reports, and view existing ones, through the
